@@ -24,15 +24,18 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <locale.h>
-#include <lcms2.h>
+#include <stdio.h>
 #include <math.h>
+#include <colord.h>
 #include <colorhug.h>
+#include <libsoup/soup.h>
 
 typedef struct {
 	ChDeviceQueue		*device_queue;
 	GOptionContext		*context;
 	GPtrArray		*cmd_array;
 	GUsbDevice		*device;
+	SoupSession		*session;
 } ChUtilPrivate;
 
 typedef gboolean (*ChUtilPrivateCb)	(ChUtilPrivate	*util,
@@ -337,6 +340,190 @@ out:
 	return ret;
 }
 
+/**
+ * ch_util_remote_profile_download:
+ **/
+static gboolean
+ch_util_remote_profile_download (ChUtilPrivate *priv, gchar **values, GError **error)
+{
+	ChSha1 remote_hash;
+	gboolean ret;
+	gchar *filename = NULL;
+	gchar *sha1 = NULL;
+	gchar *uri = NULL;
+	guint status_code;
+	SoupMessage *msg = NULL;
+	SoupURI *base_uri = NULL;
+
+	/* get the remote hash from the device */
+	ch_device_queue_get_remote_hash (priv->device_queue,
+					 priv->device,
+					 &remote_hash);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       error);
+	if (!ret)
+		goto out;
+
+	/* print hash */
+	sha1 = ch_sha1_to_string (&remote_hash);
+	uri = g_strdup_printf ("http://www.hughski.com/uploads/%s.icc", sha1);
+
+	/* GET file */
+	base_uri = soup_uri_new (uri);
+	msg = soup_message_new_from_uri (SOUP_METHOD_GET, base_uri);
+	if (msg == NULL) {
+		/* TRANSLATORS: internal error when setting up HTTP request */
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Failed to setup message");
+		goto out;
+	}
+	status_code = soup_session_send_message (priv->session, msg);
+	if (status_code != 200) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Failed to download file %s: %s",
+			     uri, msg->reason_phrase);
+		goto out;
+	}
+
+	/* copy this file into the users default icc folder */
+	filename = g_strdup_printf ("%s/%s/%s.icc",
+				    g_get_user_data_dir (),
+				    "icc",
+				    sha1);
+	ret = g_file_set_contents (filename,
+				   msg->response_body->data,
+				   msg->response_body->length,
+				   error);
+	if (!ret)
+		goto out;
+
+	/* print something */
+	if (g_strcmp0 (g_getenv ("COLORHUG_OUTPUT"), "plain") == 0)
+		g_print ("%s\n", filename);
+	else
+		g_print ("Copied remote profile into %s\n", filename);
+out:
+	if (base_uri != NULL)
+		soup_uri_free (base_uri);
+	if (msg != NULL)
+		g_object_unref (msg);
+	g_free (sha1);
+	g_free (filename);
+	g_free (uri);
+	return ret;
+}
+
+/**
+ * ch_util_self_test:
+ **/
+static gboolean
+ch_util_self_test (ChUtilPrivate *priv, gchar **values, GError **error)
+{
+	gboolean ret;
+	ch_device_queue_self_test (priv->device_queue,
+				   priv->device);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       error);
+	if (!ret)
+		goto out;
+out:
+	return ret;
+}
+
+/**
+ * ch_util_remote_profile_upload:
+ **/
+static gboolean
+ch_util_remote_profile_upload (ChUtilPrivate *priv, gchar **values, GError **error)
+{
+	ChSha1 remote_hash;
+	const gchar *uri;
+	gboolean ret = TRUE;
+	gchar *data = NULL;
+	gchar *sha1 = NULL;
+	gsize length;
+	guint status_code;
+	SoupBuffer *buffer = NULL;
+	SoupMessage *msg = NULL;
+	SoupMultipart *multipart = NULL;
+
+	/* parse */
+	if (g_strv_length (values) != 1) {
+		ret = FALSE;
+		g_set_error_literal (error, 1, 0,
+				     "invalid input, expect 'filename.icc'");
+		goto out;
+	}
+
+	/* read file */
+	ret = g_file_get_contents (values[0], &data, &length, error);
+	if (!ret)
+		goto out;
+
+	/* create multipart form and upload file */
+	multipart = soup_multipart_new (SOUP_FORM_MIME_TYPE_MULTIPART);
+	buffer = soup_buffer_new (SOUP_MEMORY_STATIC, data, length);
+	soup_multipart_append_form_file (multipart,
+					 "upload",
+					 values[0],
+					 NULL,
+					 buffer);
+	msg = soup_form_request_new_from_multipart ("http://www.hughski.com/profile-store.php", multipart);
+	status_code = soup_session_send_message (priv->session, msg);
+	if (status_code != 201) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Failed to upload file: %s",
+			     msg->reason_phrase);
+		goto out;
+	}
+	uri = soup_message_headers_get (msg->response_headers, "Location");
+	g_debug ("Successfully uploaded to %s", uri);
+
+	/* print something machine readable */
+	if (g_strcmp0 (g_getenv ("COLORHUG_OUTPUT"), "plain") == 0)
+		g_print ("%s\n", uri);
+	else
+		g_print ("Uploaded profile to %s\n", uri);
+
+	/* set SHA1 hash to device */
+	sha1 = g_compute_checksum_for_data (G_CHECKSUM_SHA1,
+					    (const guchar *) data,
+					    length);
+	g_debug ("Setting %s to device", sha1);
+	ret = ch_sha1_parse (sha1, &remote_hash, error);
+	if (!ret)
+		goto out;
+	ch_device_queue_set_remote_hash (priv->device_queue,
+					 priv->device,
+					 &remote_hash);
+	ch_device_queue_write_eeprom (priv->device_queue,
+				      priv->device,
+				      CH_WRITE_EEPROM_MAGIC);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       error);
+	if (!ret)
+		goto out;
+out:
+	if (buffer != NULL)
+		soup_buffer_free (buffer);
+	if (msg != NULL)
+		g_object_unref (msg);
+	if (multipart != NULL)
+		soup_multipart_free (multipart);
+	g_free (data);
+	g_free (sha1);
+	return ret;
+}
+
 /**
  * ch_util_set_color_select:
  **/
@@ -856,6 +1043,8 @@ ch_util_set_calibration_ccmx (ChUtilPrivate *priv, gchar **values, GError **erro
 {
 	gboolean ret;
 	guint16 calibration_index;
+	CdIt8 *ccmx = NULL;
+	GFile *file = NULL;
 
 	/* parse */
 	if (g_strv_length (values) != 2) {
@@ -869,10 +1058,15 @@ ch_util_set_calibration_ccmx (ChUtilPrivate *priv, gchar **values, GError **erro
 	calibration_index = g_ascii_strtoull (values[0], NULL, 10);
 
 	/* set to HW */
+	ccmx = cd_it8_new ();
+	file = g_file_new_for_path (values[1]);
+	ret = cd_it8_load_from_file (ccmx, file, error);
+	if (!ret)
+		goto out;
 	ret = ch_device_queue_set_calibration_ccmx (priv->device_queue,
 						    priv->device,
 						    calibration_index,
-						    values[1],
+						    ccmx,
 						    error);
 	if (!ret)
 		goto out;
@@ -883,6 +1077,10 @@ ch_util_set_calibration_ccmx (ChUtilPrivate *priv, gchar **values, GError **erro
 	if (!ret)
 		goto out;
 out:
+	if (ccmx != NULL)
+		g_object_unref (ccmx);
+	if (file != NULL)
+		g_object_unref (file);
 	return ret;
 }
 
@@ -1254,6 +1452,75 @@ out:
 	return ret;
 }
 
+/**
+ * ch_util_get_remote_hash:
+ **/
+static gboolean
+ch_util_get_remote_hash (ChUtilPrivate *priv, gchar **values, GError **error)
+{
+	ChSha1 remote_hash;
+	gboolean ret;
+	gchar *tmp = NULL;
+
+	/* get from HW */
+	ch_device_queue_get_remote_hash (priv->device_queue,
+					 priv->device,
+					 &remote_hash);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       error);
+	if (!ret)
+		goto out;
+
+	/* print hash */
+	tmp = ch_sha1_to_string (&remote_hash);
+	g_print ("%s\n", tmp);
+out:
+	g_free (tmp);
+	return ret;
+}
+
+/**
+ * ch_util_set_remote_hash:
+ **/
+static gboolean
+ch_util_set_remote_hash (ChUtilPrivate *priv, gchar **values, GError **error)
+{
+	gboolean ret;
+	ChSha1 remote_hash;
+
+	/* parse */
+	if (g_strv_length (values) != 1) {
+		ret = FALSE;
+		g_set_error_literal (error, 1, 0,
+				     "invalid input, expect sha1");
+		goto out;
+	}
+
+	/* try to parse string */
+	ret = ch_sha1_parse (values[0], &remote_hash, error);
+	if (!ret)
+		goto out;
+
+	/* set to HW */
+	ch_device_queue_set_remote_hash (priv->device_queue,
+					 priv->device,
+					 &remote_hash);
+	ch_device_queue_write_eeprom (priv->device_queue,
+				      priv->device,
+				      CH_WRITE_EEPROM_MAGIC);
+	ret = ch_device_queue_process (priv->device_queue,
+				       CH_DEVICE_QUEUE_PROCESS_FLAGS_NONE,
+				       NULL,
+				       error);
+	if (!ret)
+		goto out;
+
+out:
+	return ret;
+}
+
 /**
  * ch_util_get_dark_offsets:
  **/
@@ -1481,14 +1748,13 @@ static gboolean
 ch_util_take_readings (ChUtilPrivate *priv, gchar **values, GError **error)
 {
 	CdColorRGB value;
-	ChFreqScale multiplier = 0;
 	gboolean ret;
 	guint16 integral_time = 0;
 
 	/* get from HW */
-	ch_device_queue_get_multiplier (priv->device_queue,
+	ch_device_queue_set_multiplier (priv->device_queue,
 					priv->device,
-					&multiplier);
+					CH_FREQ_SCALE_100);
 	ch_device_queue_get_integral_time (priv->device_queue,
 					   priv->device,
 					   &integral_time);
@@ -1502,10 +1768,6 @@ ch_util_take_readings (ChUtilPrivate *priv, gchar **values, GError **error)
 	if (!ret)
 		goto out;
 
-	/* TRANSLATORS: this is the sensor scale factor */
-	g_print ("%s:\t%s\n", _("Multiplier"),
-		 ch_multiplier_to_string (multiplier));
-
 	/* TRANSLATORS: this is the sensor sample time */
 	g_print ("%s:\t0x%04x\n", _("Integral"), integral_time);
 
@@ -1594,8 +1856,12 @@ static GUsbDevice *
 ch_util_get_default_device (GError **error)
 {
 	gboolean ret;
+	GPtrArray *devices;
+	guint i;
 	GUsbContext *usb_ctx;
 	GUsbDevice *device = NULL;
+	GUsbDevice *device_success = NULL;
+	GUsbDevice *device_tmp;
 	GUsbDeviceList *list;
 
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
@@ -1604,22 +1870,42 @@ ch_util_get_default_device (GError **error)
 	usb_ctx = g_usb_context_new (NULL);
 	list = g_usb_device_list_new (usb_ctx);
 	g_usb_device_list_coldplug (list);
-	device = g_usb_device_list_find_by_vid_pid (list,
-						    CH_USB_VID,
-						    CH_USB_PID,
-						    error);
-	if (device == NULL)
+
+	/* ensure we only find one device */
+	devices = g_usb_device_list_get_devices (list);
+	for (i = 0; i < devices->len; i++) {
+		device_tmp = g_ptr_array_index (devices, i);
+		if (!ch_device_is_colorhug (device_tmp))
+			continue;
+		if (device != NULL) {
+			g_set_error_literal (error, 1, 0,
+					     _("Multiple ColorHug devices are attached"));
+			goto out;
+		}
+		device = g_object_ref (device_tmp);
+	}
+	if (device == NULL) {
+		g_set_error_literal (error, 1, 0,
+				     _("No ColorHug devices were found"));
 		goto out;
+	}
 	g_debug ("Found ColorHug device %s",
 		 g_usb_device_get_platform_id (device));
 	ret = ch_device_open (device, error);
 	if (!ret)
 		goto out;
+
+	/* success */
+	device_success = g_object_ref (device);
 out:
 	g_object_unref (usb_ctx);
+	if (device != NULL)
+		g_object_unref (device);
 	if (list != NULL)
 		g_object_unref (list);
-	return device;
+	if (devices != NULL)
+		g_ptr_array_unref (devices);
+	return device_success;
 }
 
 /**
@@ -2187,17 +2473,6 @@ ch_util_ignore_cb (const gchar *log_domain, GLogLevelFlags log_level,
 {
 }
 
-/**
- * ch_util_lcms_error_cb:
- **/
-static void
-ch_util_lcms_error_cb (cmsContext ContextID,
-		       cmsUInt32Number errorcode,
-		       const char *text)
-{
-	g_warning ("LCMS error %i: %s", errorcode, text);
-}
-
 /**
  * main:
  **/
@@ -2222,7 +2497,6 @@ main (int argc, char *argv[])
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
-	cmsSetLogErrorHandler (ch_util_lcms_error_cb);
 
 	g_type_init ();
 
@@ -2341,6 +2615,16 @@ main (int argc, char *argv[])
 		     /* TRANSLATORS: command description */
 		     _("Sets the PCB errata"),
 		     ch_util_set_pcb_errata);
+	ch_util_add (priv->cmd_array,
+		     "get-remote-hash",
+		     /* TRANSLATORS: command description */
+		     _("Gets the remote profile SHA1 hash"),
+		     ch_util_get_remote_hash);
+	ch_util_add (priv->cmd_array,
+		     "set-remote-hash",
+		     /* TRANSLATORS: command description */
+		     _("Sets the remote profile SHA1 hash"),
+		     ch_util_set_remote_hash);
 	ch_util_add (priv->cmd_array,
 		     "get-dark-offsets",
 		     /* TRANSLATORS: command description */
@@ -2451,6 +2735,21 @@ main (int argc, char *argv[])
 		     /* TRANSLATORS: command description */
 		     _("Gets an array of raw samples"),
 		     ch_util_take_reading_array);
+	ch_util_add (priv->cmd_array,
+		     "remote-profile-download",
+		     /* TRANSLATORS: command description */
+		     _("Downloads a remote profile"),
+		     ch_util_remote_profile_download);
+	ch_util_add (priv->cmd_array,
+		     "remote-profile-upload",
+		     /* TRANSLATORS: command description */
+		     _("Uploads a remote profile"),
+		     ch_util_remote_profile_upload);
+	ch_util_add (priv->cmd_array,
+		     "self-test",
+		     /* TRANSLATORS: command description */
+		     _("Does a quick self test on the device"),
+		     ch_util_self_test);
 
 	/* sort by command name */
 	g_ptr_array_sort (priv->cmd_array,
@@ -2485,6 +2784,20 @@ main (int argc, char *argv[])
 		goto out;
 	}
 
+	/* setup the session */
+	priv->session = soup_session_sync_new_with_options (SOUP_SESSION_USER_AGENT, "colorhug",
+							    SOUP_SESSION_TIMEOUT, 5000,
+							    NULL);
+	if (priv->session == NULL) {
+		/* TRANSLATORS: internal error when setting up HTTP */
+		g_print ("%s\n", _("Failed to setup networking"));
+		goto out;
+	}
+
+	/* automatically use the correct proxies */
+	soup_session_add_feature_by_type (priv->session,
+					  SOUP_TYPE_PROXY_RESOLVER_DEFAULT);
+
 	/* run the specified command */
 	ret = ch_util_run (priv, argv[1], (gchar**) &argv[2], &error);
 	if (!ret) {
@@ -2497,6 +2810,8 @@ main (int argc, char *argv[])
 	retval = 0;
 out:
 	if (priv != NULL) {
+		if (priv->session != NULL)
+			g_object_unref (priv->session);
 		if (priv->cmd_array != NULL)
 			g_ptr_array_unref (priv->cmd_array);
 		if (priv->device != NULL)
